from typing import Any

from pydantic import field_validator

from rooms_shared_services.src.models.texts.languages import Language
from rooms_shared_services.src.storage.models import BaseDynamodbModel


class TextTranslations(BaseDynamodbModel):
    source: Language = Language.en
    en: str | None = None
    ru: str | None = None
    pl: str | None = None
    he: str | None = None
    it: str | None = None
    de: str | None = None
    fr: str | None = None
    uk: str | None = None

    @field_validator("source", mode="before")
    def coerce_source(cls, item_value: Any):
        print("Item value: {}".format(item_value))
        if isinstance(item_value, str):
            try:
                return getattr(Language, item_value)
            except AttributeError:
                pass
        return item_value



from typing import Any
from rooms_shared_services.src.encoders.json import RawDynamodbEncoder
import json
from decimal import Decimal
from uuid import UUID


def convert_to_raw(data_dict: dict, root_level: bool = False) -> dict:
    return {
        item_key: convert_to_raw_value(item_value, root_level=root_level)
        for (item_key, item_value) in data_dict.items()
    }

def convert_to_raw_value(item_value: Any, root_level: bool = False) -> dict | list:
    match item_value:
        case Decimal():
            item_value_dict = {"N": str(item_value)}
        case dict():
            item_value_dict = {"M": json.dumps(item_value, cls=RawDynamodbEncoder)}
        case list():
            item_value_dict = {"L": json.dumps(item_value, cls=RawDynamodbEncoder)}
        case str():
            item_value_dict = {"S": item_value}
        case bool():
            item_value_dict = {"BOOL": "true" if item_value else "false"}
        case None:
            item_value_dict = {"NULL": ""}
        case UUID():
            item_value_dict = {"S": str(item_value)}
        case _:
            raise ValueError("Invalid item value type")
    return item_value_dict if root_level else list(item_value_dict.values()).pop()



from enum import Enum
from uuid import UUID, uuid4

from pydantic import Field, field_validator

from rooms_shared_services.src.models.texts.languages import Language
from rooms_shared_services.src.storage.models import BaseDynamodbModel


class AttributeVariant(Enum):
    FURNITURE_TYPES = "FURNITURE_TYPES"
    MATERIALS = "MATERIALS"
    COUNTRY_OF_ORIGIN = "COUNTRY_OF_ORIGIN"
    DELIVERY_TERM = "DELIVERY_TERM"


class ProductAttribute(BaseDynamodbModel):
    id: UUID = Field(default_factory=uuid4)
    attr_language: Language
    attr_name: str
    attr_terms: list[str]

    @field_validator("attr_language", mode="before")
    def attr_language(cls, field_value):
        if isinstance(field_value, str):
            try:
                return getattr(Language, field_value)
            except AttributeError:
                pass
        return field_value

    def translate_raw_term(self, raw_attr: "ProductAttribute", raw_term: str):
        term_index = raw_attr.attr_terms.index(raw_term)
        return self.attr_terms[term_index]



from pydantic_settings import BaseSettings


class Settings(BaseSettings):
    dynamodb_tablename: str = "delayed-tasks-stage"
    region_name: str = "us-east-1"
    task_variant: str = "wc_halmar_product_update"
    scheduled_tasks_timeout_sec: int = 1
    scheduled_tasks_interval_sec: int = 1
    scheduled_tasks_sleep_sec: int = 0
    pending_tasks_timeout_sec: int = 1
    pending_tasks_interval_sec: int = 1
    pending_tasks_sleep_sec: int = 0
    processing_tasks_timeout_sec: int = 1
    processing_tasks_interval_sec: int = 1
    processing_tasks_sleep_sec: int = 0
    succeded_tasks_timeout_sec: int = 1
    succeded_tasks_interval_sec: int = 1
    succeded_tasks_sleep_sec: int = 0
    failed_tasks_timeout_sec: int = 1
    failed_tasks_interval_sec: int = 1
    failed_tasks_sleep_sec: int = 0



from enum import Enum


class TextVariant(Enum):
    PRODUCT_NAME = "PRODUCT_NAME"
    PRODUCT_FULL_DESCRIPTION = "PRODUCT_FULL_DESCRIPTION"
    PRODUCT_SHORT_DESCRIPTION = "PRODUCT_SHORT_DESCRIPTION"
    PRODUCT_ATTRIBUTE_NAME = "PRODUCT_ATTRIBUTE_NAME"
    PRODUCT_ATTRIBUTE_TERM = "PRODUCT_ATTRIBUTE_TERM"
    PRODUCT_CATEGORY = "PRODUCT_CATEGORY"

    @property
    def readable(self):
        return self.value.lower().replace("_", " ")


class LLMRequestVariant(Enum):
    TRANSLATE_PRODUCT_NAME = "TRANSLATE_PRODUCT_NAME"
    TRANSLATE_PRODUCT_CATEGORY_NAME = "TRANSLATE_PRODUCT_CATEGORY_NAME"
    TRANSLATE_PRODUCT_FULL_DESCRIPTION = "TRANSLATE_PRODUCT_FULL_DESCRIPTION"
    TRANSLATE_PRODUCT_SHORT_DESCRIPTION = "TRANSLATE_PRODUCT_SHORT_DESCRIPTION"
    TRANSLATE_PRODUCT_ATTRIBUTE_NAME = "TRANSLATE_PRODUCT_ATTRIBUTE_NAME"
    TRANSLATE_PRODUCT_ATTRIBUTE_TERM = "TRANSLATE_PRODUCT_ATTRIBUTE_TERM"
    ASSIGN_PRODUCT_CATEGORY = "ASSIGN_PRODUCT_CATEGORY"
    ASSIGN_FURNITURE_TYPE = "ASSIGN_FURNITURE_TYPE"
    REQUEST_DIALOG_REPLY = "REQUEST_DIALOG_REPLY"
    IMAGINE_DIALOG_SITUATION_LOCATION = "IMAGINE_DIALOG_SITUATION_LOCATION"
    IMAGINE_DIALOG_SITUATION_PARTICIPANTS = "IMAGINE_DIALOG_SITUATION_PARTICIPANTS"
    IMAGINE_DIALOG_SITUATION_TARGETS = "IMAGINE_DIALOG_SITUATION_TARGETS"

    @property
    def readable(self):
        return self.value.replace("TRANSLATE_", "").replace("ASSIGN_", "").lower().replace("_", " ")

    @property
    def table_attr(self):
        match self.value:
            case "TRANSLATE_PRODUCT_NAME":
                return "name"
            case "TRANSLATE_PRODUCT_SHORT_DESCRIPTION":
                return "original_description_short"
            case "TRANSLATE_PRODUCT_FULL_DESCRIPTION":
                return "original_description_full"
            case _:
                raise ValueError("Invalid table attr")



from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_prefix="product_category_source_")

    tablename: str
    source_data_bucket: str
    source_data_object: str
    region_name: str
    id_index: str



import json
from datetime import datetime
from decimal import Decimal
from enum import Enum
from typing import Any, Literal
from uuid import uuid4, UUID

from pydantic import BaseModel

from rooms_shared_services.src.encoders.json import RawDynamodbEncoder

UNSET = Literal["UNSET"]


class BaseDynamodbModel(BaseModel):
    @classmethod
    def create_with_id(cls, **attributes):
        model_id = uuid4()
        value_dict = {"id": model_id, **attributes}
        return cls.model_validate(value_dict)

    def convert_value(self, item_value, exclude_unset: bool):
        match item_value:
            case BaseDynamodbModel():
                converted_item_value = item_value.dynamodb_dump(exclude_unset=exclude_unset)
            case int():
                converted_item_value = Decimal(item_value)
            case float():
                converted_item_value = Decimal(str(item_value))
            case dict():
                converted_item_value = self.convert_dict(item_value, exclude_unset=exclude_unset)
            case list():
                converted_item_value = [
                    self.convert_value(item_value_item, exclude_unset=exclude_unset) for item_value_item in item_value
                ]
            case Enum():
                converted_item_value = item_value.name
            case datetime():
                converted_item_value = item_value.isoformat()
            case _:
                converted_item_value = str(item_value)
        return converted_item_value

    def convert_to_raw(self, data_dict: dict, root_level: bool = False) -> dict:
        return {
            item_key: self.convert_to_raw_value(item_value, root_level=root_level)
            for (item_key, item_value) in data_dict.items()
        }

    def convert_to_raw_value(self, item_value: Any, root_level: bool = False):
        match item_value:
            case Decimal():
                item_value_dict = {"N": str(item_value)}
            case dict():
                item_value_dict = {"M": json.dumps(item_value, cls=RawDynamodbEncoder)}
            case list():
                item_value_dict = {"L": json.dumps(item_value, cls=RawDynamodbEncoder)}
            case str():
                item_value_dict = {"S": item_value}
            case bool():
                item_value_dict = {"BOOL": "true" if item_value else "false"}
            case None:
                item_value_dict = {"NULL": ""}
            case UUID():
                item_value_dict = {"S": str(item_value)}                
            case _:
                raise ValueError("Invalid item value type")
        return item_value_dict if root_level else list(item_value_dict.values()).pop()

    def convert_dict(self, item_dict: dict, exclude_unset: bool):
        return {
            item_key: self.convert_value(item_value, exclude_unset=exclude_unset)
            for (item_key, item_value) in item_dict.items()
        }

    def dynamodb_dump(self, exclude_unset: bool = True, include: set[str] | None = None, raw: bool = False):
        item_dict = self.model_dump(exclude_unset=exclude_unset, include=include)
        converted = self.convert_dict(item_dict=item_dict, exclude_unset=exclude_unset)
        if raw:
            converted = self.convert_to_raw(converted, root_level=True)
        return converted

    @classmethod
    def validate_value(cls, item_value: Any) -> Any:
        match item_value:
            case list():
                validated_value = [cls.validate_value(item_value=product_elem) for product_elem in item_value]
            case "None":
                validated_value = None
            case dict():
                return {
                    elem_key: cls.validate_value(item_value=elem_value) for elem_key, elem_value in item_value.items()
                }
            case _:
                validated_value = item_value
        return validated_value

    @classmethod
    def parse_raw_record_attribute(cls, attr_dict: dict[str, str]) -> Any:
        item_key, item_value = list(attr_dict.items()).pop()
        match item_key:
            case "S":
                res = str(item_value)
            case "N":
                res = float(item_value)
            case "B":
                res = item_value
            case "BOOL":
                res = item_value == "true"
            case "NULL":
                res = None
            case "M" | "L":
                res = json.loads(item_value)
            case "SS" | "NS" | "BS":
                res = set(item_value)
            case _:
                raise ValueError("Invalid data type")
        return res

    @classmethod
    def validate_dynamodb_item(cls, data_dict: dict, from_raw: bool = False):
        key_value_dict = {}
        for product_key, product_value in data_dict.items():
            validated_key = product_key
            if from_raw:
                product_value = cls.parse_raw_record_attribute(attr_dict=product_value)
            match product_key:
                case _:
                    pass
            validated_value = cls.validate_value(item_value=product_value)
            key_value_dict.update({validated_key: validated_value})

        return cls.model_validate(key_value_dict)
    
    @classmethod
    def validate_stored_item_with_none(cls, data_dict: dict, from_raw: bool = False):
        value_dict = {key: data_dict.get(key) for key in cls.model_fields}
        return cls.validate_dynamodb_item(data_dict=value_dict, from_raw=from_raw)

    @classmethod
    def validate_from_raw_record(cls, data_dict: dict):
        key_value_dict = {}
        for product_key, product_value in data_dict.items():
            validated_key = product_key
            match product_key:
                case _:
                    pass
            validated_value = cls.validate_value(item_value=product_value)
            key_value_dict.update({validated_key: validated_value})

        return cls.model_validate(key_value_dict)



import logging
from logging.handlers import HTTPHandler

from rooms_shared_services.src.logging.settings import Settings


def get_logger(name: str, use_http: bool = False, use_stream: bool = True) -> logging.Logger:
    """Get default logger.

    Args:
        name (str): _description_
        use_http (bool): _description_. Defaults to False.
        use_stream (bool): _description_. Defaults to True.

    Returns:
        logging.Logger: _description_
    """
    logger = logging.getLogger(name)
    if use_http:
        settings = Settings()  # type: ignore
        http_handler = HTTPHandler(
            settings.host,
            settings.path,
            method="POST",
            secure=settings.secure,
            context=None,
        )
        http_handler.setLevel(logging.INFO)
        http_handle_format = logging.Formatter("%(levelname)s - %(message)s - %(asctime)s")
        http_handler.setFormatter(http_handle_format)
        logger.addHandler(http_handler)
    if use_stream:
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(logging.INFO)
        stream_handle_format = logging.Formatter("%(process)s - %(message)s")
        stream_handler.setFormatter(stream_handle_format)
        logger.addHandler(stream_handler)
    return logger



from uuid import UUID

from rooms_shared_services.src.models.relations import GeoRelatedValues
from rooms_shared_services.src.storage.models import UNSET, BaseDynamodbModel


class ProductGroup(BaseDynamodbModel):
    id: UUID | UNSET = "UNSET"
    group_name: str | UNSET = "UNSET"
    brands: list[str] | UNSET = "UNSET"
    base_margins: GeoRelatedValues | UNSET = "UNSET"

    @classmethod
    def create_with_id(cls, base_margins: GeoRelatedValues | None = None, **attributes):
        if base_margins is None:
            base_margins = GeoRelatedValues()
        return super().create_with_id(base_margins=base_margins, **attributes)



from rooms_shared_services.src.delayed_tasks.abstract import AbstractQueueTaskTracker
from rooms_shared_services.src.settings.delayed_tasks import Settings as DelayedTasksSettings

delayed_tasks_settings = DelayedTasksSettings()


class DefaultTaskTracker(AbstractQueueTaskTracker):
    """Track delayed tasks of specified task variant.

    Retrieve table name from settings.

    a. Scheduled tasks.
    Receive from dynamodb table batch by batch.
    Pass to bulk processor batch by batch.
    Sleep for specified time interval.
    Repeat while scheduled tasks exist.

    b. Pending tasks.
    Receive from dynamodb table batch by batch.
    Retrieve overdue tasks.
    Mark overdue tasks as failed.

    c. Processed tasks.
    Receive from dynamodb table batch by batch.
    Retrieve overdue tasks.
    Mark overdue tasks as failed.

    d. Succeded tasks.
    Receive from dynamodb table batch by batch.
    Run success_callback.

    e. Failed tasks.
    Receive from dynamodb table batch by batch.
    Check repeat constraints.
    Schedule a repeated task as a duplicate of the failed task.

    """

    ...



from tempfile import NamedTemporaryFile

import boto3
from pydantic import create_model
from pydantic_settings import BaseSettings
from yaml import load as yaml_load

try:
    from yaml import CLoader as Loader  # noqa: WPS433
except ImportError:
    from yaml import Loader  # noqa: WPS440, WPS433
s3_client = boto3.client("s3")

with NamedTemporaryFile(suffix=".yaml") as ntf:
    s3_client.download_file("rooms-setup-data", "attributes.yaml", ntf.name)
    with open(ntf.name) as attr_fileobj:
        attributes = yaml_load(attr_fileobj.read(), Loader=Loader)
        furniture_types = attributes["furniture_types"]


FurnitureTypeAttributeCollection = create_model(
    "FurnitureTypeAttributeCollection",
    **{furniture_type: (int, ...) for furniture_type in furniture_types},
)


class PriceMarginSettings(BaseSettings, FurnitureTypeAttributeCollection):
    ...
